"""
This module provides Python wrappers for the GPU implementation in Blender.
Some higher level functions can be found in the gpu_extras module.

gpu.types.rst
gpu.matrix.rst
gpu.select.rst
gpu.shader.rst
gpu.state.rst
gpu.texture.rst
gpu.platform.rst
gpu.capabilities.rst

:maxdepth: 1
:caption: Submodules


--------------------

Geometry is drawn in batches.
A batch contains the necessary data to perform the drawing.
That includes an obligatory *Vertex Buffer* and an optional *Index Buffer*,
each of which is described in more detail in the following sections.
A batch also defines a draw type.
Typical draw types are POINTS

, LINES

 and TRIS

.
The draw type determines how the data will be interpreted and drawn.


--------------------

A *Vertex Buffer Object* (VBO) (gpu.types.GPUVertBuf)
is an array that contains the vertex attributes needed for drawing using a specific shader.
Typical vertex attributes are *location*, *normal*, *color*, and *uv*.
Every vertex buffer has a *Vertex Format* (gpu.types.GPUVertFormat)
and a length corresponding to the number of vertices in the buffer.
A vertex format describes the attributes stored per vertex and their types.

The following code demonstrates the creation of a vertex buffer that contains 6 vertices.
For each vertex 2 attributes will be stored: The position and the normal.

```
import gpu
vertex_positions = [(0, 0, 0), ...]
vertex_normals = [(0, 0, 1), ...]

fmt = gpu.types.GPUVertFormat()
fmt.attr_add(id="pos", comp_type='F32', len=3, fetch_mode='FLOAT')
fmt.attr_add(id="normal", comp_type='F32', len=3, fetch_mode='FLOAT')

vbo = gpu.types.GPUVertBuf(len=6, format=fmt)
vbo.attr_fill(id="pos", data=vertex_positions)
vbo.attr_fill(id="normal", data=vertex_normals)
```

This vertex buffer could be used to draw 6 points, 3 separate lines, 5 consecutive lines, 2 separate triangles, ...
E.g. in the case of lines, each two consecutive vertices define a line.
The type that will actually be drawn is determined when the batch is created later.


--------------------

Often triangles and lines share one or more vertices.
With only a vertex buffer one would have to store all attributes for the these vertices multiple times.
This is very inefficient because in a connected triangle mesh every vertex is used 6 times on average.
A more efficient approach would be to use an *Index Buffer* (IBO) (gpu.types.GPUIndexBuf),
sometimes referred to as *Element Buffer*.
An *Index Buffer* is an array that references vertices based on their index in the vertex buffer.

For instance, to draw a rectangle composed of two triangles, one could use an index buffer.

```
positions = (
    (-1,  1), (1,  1),
    (-1, -1), (1, -1))

indices = ((0, 1, 2), (2, 1, 3))

ibo = gpu.types.GPUIndexBuf(type='TRIS', seq=indices)
```

Here the first tuple in indices

 describes which vertices should be used for the first triangle
(same for the second tuple).
Note how the diagonal vertices 1 and 2 are shared between both triangles.


--------------------

A shader is a program that runs on the GPU (written in GLSL in our case).
There are multiple types of shaders.
The most important ones are *Vertex Shaders* and *Fragment Shaders*.
Typically multiple shaders are linked together into a *Program*.
However, in the Blender Python API the term *Shader* refers to an OpenGL Program.
Every gpu.types.GPUShader consists of a vertex shader, a fragment shader and an optional geometry shader.
For common drawing tasks there are some built-in shaders accessible from gpu.shader.from_builtin
with an identifier such as UNIFORM_COLOR

 or FLAT_COLOR

.

Every shader defines a set of attributes and uniforms that have to be set in order to use the shader.
Attributes are properties that are set using a vertex buffer and can be different for individual vertices.
Uniforms are properties that are constant per draw call.
They can be set using the shader.uniform_*

 functions after the shader has been bound.


--------------------

Batches can be creates by first manually creating VBOs and IBOs.
However, it is recommended to use the gpu_extras.batch.batch_for_shader function.
It makes sure that all the vertex attributes necessary for a specific shader are provided.
Consequently, the shader has to be passed to the function as well.
When using this function one rarely has to care about the vertex format, VBOs and IBOs created in the background.
This is still something one should know when drawing stuff though.

Since batches can be drawn multiple times, they should be cached and reused whenever possible.


--------------------

What one can see on the screen after rendering is called the *Front Buffer*.
When draw calls are issued, batches are drawn on a *Back Buffer* that will only be displayed
when all drawing is done and the current back buffer will become the new front buffer.
Sometimes, one might want to draw the batches into a distinct buffer that could be used as
texture to display on another object or to be saved as image on disk.
This is called Offscreen Rendering.
In Blender Offscreen Rendering is done using the gpu.types.GPUOffScreen type.

[WARNING]
gpu.types.GPUOffScreen objects are bound to the OpenGL context they have been created in.
This means that once Blender discards this context (i.e. the window is closed),
the offscreen instance will be freed.


--------------------

To try these examples, just copy them into Blenders text editor and execute them.
To keep the examples relatively small, they just register a draw function that can't easily be removed anymore.
Blender has to be restarted in order to delete the draw handlers.


--------------------

```../examples/gpu.1.py```


--------------------

```../examples/gpu.2.py```


--------------------

```../examples/gpu.3.py```


--------------------

```../examples/gpu.4.py```


--------------------

```../examples/gpu.5.py```


--------------------

To use this example you have to provide an image that should be displayed.

```../examples/gpu.6.py```


--------------------

1. Create an gpu.types.GPUOffScreen object.
2. Draw some circles into it.
3. Make a new shader for drawing a planar texture in 3D.
4. Draw the generated texture using the new shader.

```../examples/gpu.7.py```


--------------------

This will create a new image with the given name.
If it already exists, it will override the existing one.

Currently almost all of the execution time is spent in the last line.
In the future this will hopefully be solved by implementing the Python buffer protocol
for gpu.types.Buffer and bpy.types.Image.pixels (aka bpy_prop_array

).

```../examples/gpu.8.py```


--------------------

The scene has to have a camera for this example to work.
You could also make this independent of a specific camera,
but Blender does not expose good functions to create view and projection matrices yet.

```../examples/gpu.9.py```


--------------------

In this example the arc length (distance to the first point on the line) is calculated in every vertex.
Between the vertex and fragment shader that value is automatically interpolated
for all points that will be visible on the screen.
In the fragment shader the sin

 of the arc length is calculated.
Based on the result a decision is made on whether the fragment should be drawn or not.

```../examples/gpu.10.py```


--------------------

This is an example of how to use a custom compute shader to write to a texture and then use that texture in a vertex/fragment shader.
The expected result is a 2x2 plane (size of the default cube), which changes color from a green-black gradient to a green-red gradient,
based on current time.

```../examples/gpu.11.py```

"""

import typing
import collections.abc
import typing_extensions
from . import capabilities as capabilities
from . import matrix as matrix
from . import platform as platform
from . import select as select
from . import shader as shader
from . import state as state
from . import texture as texture
from . import types as types



"""
This module provides access to math operations.

[NOTE]
Classes, methods and attributes that accept vectors also accept other numeric sequences,
such as tuples, lists.

The mathutils module provides the following classes:

* Color,
* Euler,
* Matrix,
* Quaternion,
* Vector,

mathutils.geometry.rst
mathutils.bvhtree.rst
mathutils.kdtree.rst
mathutils.interpolate.rst
mathutils.noise.rst

:maxdepth: 1
:caption: Submodules

```../examples/mathutils.py```

"""

import typing
import collections.abc
import typing_extensions
from . import bvhtree as bvhtree
from . import geometry as geometry
from . import interpolate as interpolate
from . import kdtree as kdtree
from . import noise as noise

class Color:
    """This object gives access to Colors in Blender.Most colors returned by Blender APIs are in scene linear color space, as defined by    the OpenColorIO configuration. The notable exception is user interface theming colors,    which are in sRGB color space."""

    b: float
    """ Blue color channel.

    :type: float
    """

    g: float
    """ Green color channel.

    :type: float
    """

    h: float
    """ HSV Hue component in [0, 1].

    :type: float
    """

    hsv: Vector | collections.abc.Sequence[float]
    """ HSV Values in [0, 1].

    :type: Vector | collections.abc.Sequence[float]
    """

    is_frozen: bool
    """ True when this object has been frozen (read-only).

    :type: bool
    """

    is_valid: bool
    """ True when the owner of this data is valid.

    :type: bool
    """

    is_wrapped: bool
    """ True when this object wraps external data (read-only).

    :type: bool
    """

    owner: typing.Any
    """ The item this is wrapping or None  (read-only)."""

    r: float
    """ Red color channel.

    :type: float
    """

    s: float
    """ HSV Saturation component in [0, 1].

    :type: float
    """

    v: float
    """ HSV Value component in [0, 1].

    :type: float
    """

    def copy(self) -> typing_extensions.Self:
        """Returns a copy of this color.

        :return: A copy of the color.
        :rtype: typing_extensions.Self
        """

    def freeze(self) -> typing_extensions.Self:
        """Make this object immutable.After this the object can be hashed, used in dictionaries & sets.

        :return: An instance of this object.
        :rtype: typing_extensions.Self
        """

    def from_aces_to_scene_linear(self) -> typing_extensions.Self:
        """Convert from ACES2065-1 linear to scene linear color space.

        :return: A color in scene linear color space.
        :rtype: typing_extensions.Self
        """

    def from_rec709_linear_to_scene_linear(self) -> typing_extensions.Self:
        """Convert from Rec.709 linear color space to scene linear color space.

        :return: A color in scene linear color space.
        :rtype: typing_extensions.Self
        """

    def from_scene_linear_to_aces(self) -> typing_extensions.Self:
        """Convert from scene linear to ACES2065-1 linear color space.

        :return: A color in ACES2065-1 linear color space.
        :rtype: typing_extensions.Self
        """

    def from_scene_linear_to_rec709_linear(self) -> typing_extensions.Self:
        """Convert from scene linear to Rec.709 linear color space.

        :return: A color in Rec.709 linear color space.
        :rtype: typing_extensions.Self
        """

    def from_scene_linear_to_srgb(self) -> typing_extensions.Self:
        """Convert from scene linear to sRGB color space.

        :return: A color in sRGB color space.
        :rtype: typing_extensions.Self
        """

    def from_scene_linear_to_xyz_d65(self) -> typing_extensions.Self:
        """Convert from scene linear to CIE XYZ (Illuminant D65) color space.

        :return: A color in XYZ color space.
        :rtype: typing_extensions.Self
        """

    def from_srgb_to_scene_linear(self) -> typing_extensions.Self:
        """Convert from sRGB to scene linear color space.

        :return: A color in scene linear color space.
        :rtype: typing_extensions.Self
        """

    def from_xyz_d65_to_scene_linear(self) -> typing_extensions.Self:
        """Convert from CIE XYZ (Illuminant D65) to scene linear color space.

        :return: A color in scene linear color space.
        :rtype: typing_extensions.Self
        """

    def __init__(self, rgb: collections.abc.Sequence[float] = (0.0, 0.0, 0.0)):
        """

        :param rgb:
        :type rgb: collections.abc.Sequence[float]
        """

    def __get__(self, instance, owner) -> typing_extensions.Self:
        """

        :param instance:
        :param owner:
        :return:
        :rtype: typing_extensions.Self
        """

    def __set__(
        self, instance, value: collections.abc.Sequence[float] | typing_extensions.Self
    ):
        """

        :param instance:
        :param value:
        :type value: collections.abc.Sequence[float] | typing_extensions.Self
        """

    def __add__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :type other: collections.abc.Sequence[float] | typing_extensions.Self
        :return:
        :rtype: typing_extensions.Self
        """

    def __sub__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :type other: collections.abc.Sequence[float] | typing_extensions.Self
        :return:
        :rtype: typing_extensions.Self
        """

    def __mul__(self, other: float) -> typing_extensions.Self:
        """

        :param other:
        :type other: float
        :return:
        :rtype: typing_extensions.Self
        """

    def __truediv__(self, other: float) -> typing_extensions.Self:
        """

        :param other:
        :type other: float
        :return:
        :rtype: typing_extensions.Self
        """

    def __radd__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :type other: collections.abc.Sequence[float] | typing_extensions.Self
        :return:
        :rtype: typing_extensions.Self
        """

    def __rsub__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :type other: collections.abc.Sequence[float] | typing_extensions.Self
        :return:
        :rtype: typing_extensions.Self
        """

    def __rmul__(self, other: float) -> typing_extensions.Self:
        """

        :param other:
        :type other: float
        :return:
        :rtype: typing_extensions.Self
        """

    def __rtruediv__(self, other: float) -> typing_extensions.Self:
        """

        :param other:
        :type other: float
        :return:
        :rtype: typing_extensions.Self
        """

    def __iadd__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :type other: collections.abc.Sequence[float] | typing_extensions.Self
        :return:
        :rtype: typing_extensions.Self
        """

    def __isub__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :type other: collections.abc.Sequence[float] | typing_extensions.Self
        :return:
        :rtype: typing_extensions.Self
        """

    def __imul__(self, other: float) -> typing_extensions.Self:
        """

        :param other:
        :type other: float
        :return:
        :rtype: typing_extensions.Self
        """

    def __itruediv__(self, other: float) -> typing_extensions.Self:
        """

        :param other:
        :type other: float
        :return:
        :rtype: typing_extensions.Self
        """

    @typing.overload
    def __getitem__(self, key: int) -> float:
        """

        :param key:
        :type key: int
        :return:
        :rtype: float
        """

    @typing.overload
    def __getitem__(self, key: slice) -> tuple[float, ...]:
        """

        :param key:
        :type key: slice
        :return:
        :rtype: tuple[float, ...]
        """

    @typing.overload
    def __setitem__(self, key: int, value: float):
        """

        :param key:
        :type key: int
        :param value:
        :type value: float
        """

    @typing.overload
    def __setitem__(self, key: slice, value: collections.abc.Iterable[float] | Color):
        """

        :param key:
        :type key: slice
        :param value:
        :type value: collections.abc.Iterable[float] | Color
        """

class Euler:
    """This object gives access to Eulers in Blender.`Euler angles <https://en.wikipedia.org/wiki/Euler_angles>`__ on Wikipedia."""

    is_frozen: bool
    """ True when this object has been frozen (read-only).

    :type: bool
    """

    is_valid: bool
    """ True when the owner of this data is valid.

    :type: bool
    """

    is_wrapped: bool
    """ True when this object wraps external data (read-only).

    :type: bool
    """

    order: typing.Any
    """ Euler rotation order."""

    owner: typing.Any
    """ The item this is wrapping or None  (read-only)."""

    x: float
    """ Euler axis angle in radians.

    :type: float
    """

    y: float
    """ Euler axis angle in radians.

    :type: float
    """

    z: float
    """ Euler axis angle in radians.

    :type: float
    """

    def copy(self) -> typing_extensions.Self:
        """Returns a copy of this euler.

        :return: A copy of the euler.
        :rtype: typing_extensions.Self
        """

    def freeze(self) -> typing_extensions.Self:
        """Make this object immutable.After this the object can be hashed, used in dictionaries & sets.

        :return: An instance of this object.
        :rtype: typing_extensions.Self
        """

    def make_compatible(self, other):
        """Make this euler compatible with another,
        so interpolating between them works as intended.

                :param other:
        """

    def rotate(
        self,
        other: Matrix
        | Quaternion
        | collections.abc.Sequence[collections.abc.Sequence[float]]
        | collections.abc.Sequence[float]
        | typing_extensions.Self,
    ):
        """Rotates the euler by another mathutils value.

        :param other: rotation component of mathutils value
        :type other: Matrix | Quaternion | collections.abc.Sequence[collections.abc.Sequence[float]] | collections.abc.Sequence[float] | typing_extensions.Self
        """

    def rotate_axis(self, axis: str, angle: float):
        """Rotates the euler a certain amount and returning a unique euler rotation
        (no 720 degree pitches).

                :param axis: single character in ['X, 'Y', 'Z'].
                :type axis: str
                :param angle: angle in radians.
                :type angle: float
        """

    def to_matrix(self) -> Matrix:
        """Return a matrix representation of the euler.

        :return: A 3x3 rotation matrix representation of the euler.
        :rtype: Matrix
        """

    def to_quaternion(self) -> Quaternion:
        """Return a quaternion representation of the euler.

        :return: Quaternion representation of the euler.
        :rtype: Quaternion
        """

    def zero(self):
        """Set all values to zero."""

    def __init__(
        self,
        angles: collections.abc.Sequence[float] = (0.0, 0.0, 0.0),
        order: str = "XYZ",
    ):
        """

        :param angles:
        :type angles: collections.abc.Sequence[float]
        :param order:
        :type order: str
        """

    def __get__(self, instance, owner) -> typing_extensions.Self:
        """

        :param instance:
        :param owner:
        :return:
        :rtype: typing_extensions.Self
        """

    def __set__(
        self, instance, value: collections.abc.Sequence[float] | typing_extensions.Self
    ):
        """

        :param instance:
        :param value:
        :type value: collections.abc.Sequence[float] | typing_extensions.Self
        """

    @typing.overload
    def __getitem__(self, key: int) -> float:
        """

        :param key:
        :type key: int
        :return:
        :rtype: float
        """

    @typing.overload
    def __getitem__(self, key: slice) -> tuple[float, ...]:
        """

        :param key:
        :type key: slice
        :return:
        :rtype: tuple[float, ...]
        """

    @typing.overload
    def __setitem__(self, key: int, value: float):
        """

        :param key:
        :type key: int
        :param value:
        :type value: float
        """

    @typing.overload
    def __setitem__(self, key: slice, value: collections.abc.Iterable[float] | Euler):
        """

        :param key:
        :type key: slice
        :param value:
        :type value: collections.abc.Iterable[float] | Euler
        """

class Matrix:
    """This object gives access to Matrices in Blender, supporting square and rectangular
    matrices from 2x2 up to 4x4.
    """

    col: typing.Any
    """ Access the matrix by columns, 3x3 and 4x4 only, (read-only)."""

    is_frozen: bool
    """ True when this object has been frozen (read-only).

    :type: bool
    """

    is_identity: bool
    """ True if this is an identity matrix (read-only).

    :type: bool
    """

    is_negative: bool
    """ True if this matrix results in a negative scale, 3x3 and 4x4 only, (read-only).

    :type: bool
    """

    is_orthogonal: bool
    """ True if this matrix is orthogonal, 3x3 and 4x4 only, (read-only).

    :type: bool
    """

    is_orthogonal_axis_vectors: bool
    """ True if this matrix has got orthogonal axis vectors, 3x3 and 4x4 only, (read-only).

    :type: bool
    """

    is_valid: bool
    """ True when the owner of this data is valid.

    :type: bool
    """

    is_wrapped: bool
    """ True when this object wraps external data (read-only).

    :type: bool
    """

    median_scale: float
    """ The average scale applied to each axis (read-only).

    :type: float
    """

    owner: typing.Any
    """ The item this is wrapping or None  (read-only)."""

    row: typing.Any
    """ Access the matrix by rows (default), (read-only)."""

    translation: Vector
    """ The translation component of the matrix.

    :type: Vector
    """

    @classmethod
    def Diagonal(
        cls, vector: Vector | collections.abc.Sequence[float]
    ) -> typing_extensions.Self:
        """Create a diagonal (scaling) matrix using the values from the vector.

        :param vector: The vector of values for the diagonal.
        :type vector: Vector | collections.abc.Sequence[float]
        :return: A diagonal matrix.
        :rtype: typing_extensions.Self
        """

    @classmethod
    def Identity(cls, size: int) -> typing_extensions.Self:
        """Create an identity matrix.

        :param size: The size of the identity matrix to construct [2, 4].
        :type size: int
        :return: A new identity matrix.
        :rtype: typing_extensions.Self
        """

    @classmethod
    def LocRotScale(
        cls,
        location: None | Vector | collections.abc.Sequence[float],
        rotation: Euler
        | None
        | Quaternion
        | collections.abc.Sequence[collections.abc.Sequence[float]]
        | collections.abc.Sequence[float]
        | typing_extensions.Self,
        scale: None | Vector | collections.abc.Sequence[float],
    ) -> typing_extensions.Self:
        """Create a matrix combining translation, rotation and scale,
        acting as the inverse of the decompose() method.Any of the inputs may be replaced with None if not needed.

                :param location: The translation component.
                :type location: None | Vector | collections.abc.Sequence[float]
                :param rotation: The rotation component as a 3x3 matrix, quaternion, euler or None for no rotation.
                :type rotation: Euler | None | Quaternion | collections.abc.Sequence[collections.abc.Sequence[float]] | collections.abc.Sequence[float] | typing_extensions.Self
                :param scale: The scale component.
                :type scale: None | Vector | collections.abc.Sequence[float]
                :return: Combined transformation as a 4x4 matrix.
                :rtype: typing_extensions.Self
        """

    @classmethod
    def OrthoProjection(
        cls, axis: Vector | collections.abc.Sequence[float] | str, size: int
    ) -> typing_extensions.Self:
        """Create a matrix to represent an orthographic projection.

                :param axis: Can be any of the following: ['X', 'Y', 'XY', 'XZ', 'YZ'],
        where a single axis is for a 2D matrix.
        Or a vector for an arbitrary axis
                :type axis: Vector | collections.abc.Sequence[float] | str
                :param size: The size of the projection matrix to construct [2, 4].
                :type size: int
                :return: A new projection matrix.
                :rtype: typing_extensions.Self
        """

    @classmethod
    def Rotation(
        cls,
        angle: float,
        size: int,
        axis: Vector | collections.abc.Sequence[float] | str | None = "",
    ) -> typing_extensions.Self:
        """Create a matrix representing a rotation.

                :param angle: The angle of rotation desired, in radians.
                :type angle: float
                :param size: The size of the rotation matrix to construct [2, 4].
                :type size: int
                :param axis: a string in ['X', 'Y', 'Z'] or a 3D Vector Object
        (optional when size is 2).
                :type axis: Vector | collections.abc.Sequence[float] | str | None
                :return: A new rotation matrix.
                :rtype: typing_extensions.Self
        """

    @classmethod
    def Scale(
        cls,
        factor: float,
        size: int,
        axis: Vector | collections.abc.Sequence[float] | None = [],
    ) -> typing_extensions.Self:
        """Create a matrix representing a scaling.

        :param factor: The factor of scaling to apply.
        :type factor: float
        :param size: The size of the scale matrix to construct [2, 4].
        :type size: int
        :param axis: Direction to influence scale. (optional).
        :type axis: Vector | collections.abc.Sequence[float] | None
        :return: A new scale matrix.
        :rtype: typing_extensions.Self
        """

    @classmethod
    def Shear(
        cls, plane: str, size: int, factor: collections.abc.Sequence[float] | float
    ) -> typing_extensions.Self:
        """Create a matrix to represent an shear transformation.

                :param plane: Can be any of the following: ['X', 'Y', 'XY', 'XZ', 'YZ'],
        where a single axis is for a 2D matrix only.
                :type plane: str
                :param size: The size of the shear matrix to construct [2, 4].
                :type size: int
                :param factor: The factor of shear to apply. For a 2 size matrix use a single float. For a 3 or 4 size matrix pass a pair of floats corresponding with the plane axis.
                :type factor: collections.abc.Sequence[float] | float
                :return: A new shear matrix.
                :rtype: typing_extensions.Self
        """

    @classmethod
    def Translation(
        cls, vector: Vector | collections.abc.Sequence[float]
    ) -> typing_extensions.Self:
        """Create a matrix representing a translation.

        :param vector: The translation vector.
        :type vector: Vector | collections.abc.Sequence[float]
        :return: An identity matrix with a translation.
        :rtype: typing_extensions.Self
        """

    def adjugate(self):
        """Set the matrix to its adjugate.`Adjugate matrix <https://en.wikipedia.org/wiki/Adjugate_matrix>`__ on Wikipedia."""

    def adjugated(self) -> typing_extensions.Self:
        """Return an adjugated copy of the matrix.

        :return: the adjugated matrix.
        :rtype: typing_extensions.Self
        """

    def copy(self) -> typing_extensions.Self:
        """Returns a copy of this matrix.

        :return: an instance of itself
        :rtype: typing_extensions.Self
        """

    def decompose(self) -> tuple[Vector, Quaternion, Vector]:
        """Return the translation, rotation, and scale components of this matrix.

        :return: Tuple of translation, rotation, and scale.
        :rtype: tuple[Vector, Quaternion, Vector]
        """

    def determinant(self) -> float:
        """Return the determinant of a matrix.`Determinant <https://en.wikipedia.org/wiki/Determinant>`__ on Wikipedia.

        :return: Return the determinant of a matrix.
        :rtype: float
        """

    def freeze(self) -> typing_extensions.Self:
        """Make this object immutable.After this the object can be hashed, used in dictionaries & sets.

        :return: An instance of this object.
        :rtype: typing_extensions.Self
        """

    def identity(self):
        """Set the matrix to the identity matrix.`Identity matrix <https://en.wikipedia.org/wiki/Identity_matrix>`__ on Wikipedia."""

    def invert(
        self,
        fallback: collections.abc.Sequence[collections.abc.Sequence[float]]
        | typing_extensions.Self
        | None = None,
    ):
        """Set the matrix to its inverse.`Inverse matrix <https://en.wikipedia.org/wiki/Inverse_matrix>`__ on Wikipedia.

                :param fallback: Set the matrix to this value when the inverse cannot be calculated
        (instead of raising a `ValueError` exception).
                :type fallback: collections.abc.Sequence[collections.abc.Sequence[float]] | typing_extensions.Self | None
        """

    def invert_safe(self):
        """Set the matrix to its inverse, will never error.
        If degenerated (e.g. zero scale on an axis), add some epsilon to its diagonal, to get an invertible one.
        If tweaked matrix is still degenerated, set to the identity matrix instead.`Inverse Matrix <https://en.wikipedia.org/wiki/Inverse_matrix>`__ on Wikipedia.

        """

    def inverted(
        self, fallback: typing.Any | None = None
    ) -> typing.Any | typing_extensions.Self:
        """Return an inverted copy of the matrix.

                :param fallback: return this when the inverse can't be calculated
        (instead of raising a `ValueError`).
                :type fallback: typing.Any | None
                :return: The inverted matrix or fallback when given.
                :rtype: typing.Any | typing_extensions.Self
        """

    def inverted_safe(self) -> typing_extensions.Self:
        """Return an inverted copy of the matrix, will never error.
        If degenerated (e.g. zero scale on an axis), add some epsilon to its diagonal, to get an invertible one.
        If tweaked matrix is still degenerated, return the identity matrix instead.

                :return: the inverted matrix.
                :rtype: typing_extensions.Self
        """

    def lerp(
        self,
        other: collections.abc.Sequence[collections.abc.Sequence[float]]
        | typing_extensions.Self,
        factor: float,
    ) -> typing_extensions.Self:
        """Returns the interpolation of two matrices. Uses polar decomposition, see   "Matrix Animation and Polar Decomposition", Shoemake and Duff, 1992.

        :param other: value to interpolate with.
        :type other: collections.abc.Sequence[collections.abc.Sequence[float]] | typing_extensions.Self
        :param factor: The interpolation value in [0.0, 1.0].
        :type factor: float
        :return: The interpolated matrix.
        :rtype: typing_extensions.Self
        """

    def normalize(self):
        """Normalize each of the matrix columns."""

    def normalized(self) -> typing_extensions.Self:
        """Return a column normalized matrix

        :return: a column normalized matrix
        :rtype: typing_extensions.Self
        """

    def resize_4x4(self):
        """Resize the matrix to 4x4."""

    def rotate(
        self,
        other: Euler
        | Quaternion
        | collections.abc.Sequence[collections.abc.Sequence[float]]
        | collections.abc.Sequence[float]
        | typing_extensions.Self,
    ):
        """Rotates the matrix by another mathutils value.

        :param other: rotation component of mathutils value
        :type other: Euler | Quaternion | collections.abc.Sequence[collections.abc.Sequence[float]] | collections.abc.Sequence[float] | typing_extensions.Self
        """

    def to_2x2(self) -> typing_extensions.Self:
        """Return a 2x2 copy of this matrix.

        :return: a new matrix.
        :rtype: typing_extensions.Self
        """

    def to_3x3(self) -> typing_extensions.Self:
        """Return a 3x3 copy of this matrix.

        :return: a new matrix.
        :rtype: typing_extensions.Self
        """

    def to_4x4(self) -> typing_extensions.Self:
        """Return a 4x4 copy of this matrix.

        :return: a new matrix.
        :rtype: typing_extensions.Self
        """

    def to_euler(
        self,
        order: str | None = "",
        euler_compat: Euler | collections.abc.Sequence[float] | None = [],
    ) -> Euler:
        """Return an Euler representation of the rotation matrix
        (3x3 or 4x4 matrix only).

                :param order: Optional rotation order argument in
        ['XYZ', 'XZY', 'YXZ', 'YZX', 'ZXY', 'ZYX'].
                :type order: str | None
                :param euler_compat: Optional euler argument the new euler will be made
        compatible with (no axis flipping between them).
        Useful for converting a series of matrices to animation curves.
                :type euler_compat: Euler | collections.abc.Sequence[float] | None
                :return: Euler representation of the matrix.
                :rtype: Euler
        """

    def to_quaternion(self) -> Quaternion:
        """Return a quaternion representation of the rotation matrix.

        :return: Quaternion representation of the rotation matrix.
        :rtype: Quaternion
        """

    def to_scale(self) -> Vector:
        """Return the scale part of a 3x3 or 4x4 matrix.

        :return: Return the scale of a matrix.
        :rtype: Vector
        """

    def to_translation(self) -> Vector:
        """Return the translation part of a 4 row matrix.

        :return: Return the translation of a matrix.
        :rtype: Vector
        """

    def transpose(self):
        """Set the matrix to its transpose.`Transpose <https://en.wikipedia.org/wiki/Transpose>`__ on Wikipedia."""

    def transposed(self) -> typing_extensions.Self:
        """Return a new, transposed matrix.

        :return: a transposed matrix
        :rtype: typing_extensions.Self
        """

    def zero(self):
        """Set all the matrix values to zero."""

    def __init__(
        self,
        rows: collections.abc.Sequence[collections.abc.Sequence[float]] = (
            (1.0, 0.0, 0.0, 0.0),
            (0.0, 1.0, 0.0, 0.0),
            (0.0, 0.0, 1.0, 0.0),
            (0.0, 0.0, 0.0, 1.0),
        ),
    ):
        """

        :param rows:
        :type rows: collections.abc.Sequence[collections.abc.Sequence[float]]
        """

    def __get__(self, instance, owner) -> typing_extensions.Self:
        """

        :param instance:
        :param owner:
        :return:
        :rtype: typing_extensions.Self
        """

    def __set__(
        self,
        instance,
        value: collections.abc.Sequence[collections.abc.Sequence[float]]
        | typing_extensions.Self,
    ):
        """

        :param instance:
        :param value:
        :type value: collections.abc.Sequence[collections.abc.Sequence[float]] | typing_extensions.Self
        """

    @typing.overload
    def __getitem__(self, key: int) -> Vector:
        """

        :param key:
        :type key: int
        :return:
        :rtype: Vector
        """

    @typing.overload
    def __getitem__(self, key: slice) -> tuple[Vector, ...]:
        """

        :param key:
        :type key: slice
        :return:
        :rtype: tuple[Vector, ...]
        """

    @typing.overload
    def __setitem__(self, key: int, value: Vector | collections.abc.Iterable[float]):
        """

        :param key:
        :type key: int
        :param value:
        :type value: Vector | collections.abc.Iterable[float]
        """

    @typing.overload
    def __setitem__(
        self,
        key: slice,
        value: collections.abc.Iterable[Vector | collections.abc.Iterable[float]]
        | Matrix,
    ):
        """

        :param key:
        :type key: slice
        :param value:
        :type value: collections.abc.Iterable[Vector | collections.abc.Iterable[float]] | Matrix
        """

    def __len__(self) -> int:
        """

        :return:
        :rtype: int
        """

    def __add__(
        self,
        other: collections.abc.Sequence[collections.abc.Sequence[float]]
        | typing_extensions.Self,
    ) -> typing_extensions.Self:
        """

        :param other:
        :type other: collections.abc.Sequence[collections.abc.Sequence[float]] | typing_extensions.Self
        :return:
        :rtype: typing_extensions.Self
        """

    def __sub__(
        self,
        other: collections.abc.Sequence[collections.abc.Sequence[float]]
        | typing_extensions.Self,
    ) -> typing_extensions.Self:
        """

        :param other:
        :type other: collections.abc.Sequence[collections.abc.Sequence[float]] | typing_extensions.Self
        :return:
        :rtype: typing_extensions.Self
        """

    def __mul__(self, other: float) -> typing_extensions.Self:
        """

        :param other:
        :type other: float
        :return:
        :rtype: typing_extensions.Self
        """

    @typing.overload
    def __matmul__(
        self,
        other: collections.abc.Sequence[collections.abc.Sequence[float]]
        | typing_extensions.Self,
    ) -> typing_extensions.Self:
        """

        :param other:
        :type other: collections.abc.Sequence[collections.abc.Sequence[float]] | typing_extensions.Self
        :return:
        :rtype: typing_extensions.Self
        """

    @typing.overload
    def __matmul__(self, other: Vector | collections.abc.Sequence[float]) -> Vector:
        """

        :param other:
        :type other: Vector | collections.abc.Sequence[float]
        :return:
        :rtype: Vector
        """

    def __radd__(
        self,
        other: collections.abc.Sequence[collections.abc.Sequence[float]]
        | typing_extensions.Self,
    ) -> typing_extensions.Self:
        """

        :param other:
        :type other: collections.abc.Sequence[collections.abc.Sequence[float]] | typing_extensions.Self
        :return:
        :rtype: typing_extensions.Self
        """

    def __rsub__(
        self,
        other: collections.abc.Sequence[collections.abc.Sequence[float]]
        | typing_extensions.Self,
    ) -> typing_extensions.Self:
        """

        :param other:
        :type other: collections.abc.Sequence[collections.abc.Sequence[float]] | typing_extensions.Self
        :return:
        :rtype: typing_extensions.Self
        """

    def __rmul__(self, other: float) -> typing_extensions.Self:
        """

        :param other:
        :type other: float
        :return:
        :rtype: typing_extensions.Self
        """

    def __imul__(self, other: float) -> typing_extensions.Self:
        """

        :param other:
        :type other: float
        :return:
        :rtype: typing_extensions.Self
        """

class Quaternion:
    """This object gives access to Quaternions in Blender.The constructor takes arguments in various forms:"""

    angle: float
    """ Angle of the quaternion.

    :type: float
    """

    axis: Vector
    """ Quaternion axis as a vector.

    :type: Vector
    """

    is_frozen: bool
    """ True when this object has been frozen (read-only).

    :type: bool
    """

    is_valid: bool
    """ True when the owner of this data is valid.

    :type: bool
    """

    is_wrapped: bool
    """ True when this object wraps external data (read-only).

    :type: bool
    """

    magnitude: float
    """ Size of the quaternion (read-only).

    :type: float
    """

    owner: typing.Any
    """ The item this is wrapping or None  (read-only)."""

    w: float
    """ Quaternion axis value.

    :type: float
    """

    x: float
    """ Quaternion axis value.

    :type: float
    """

    y: float
    """ Quaternion axis value.

    :type: float
    """

    z: float
    """ Quaternion axis value.

    :type: float
    """

    def conjugate(self):
        """Set the quaternion to its conjugate (negate x, y, z)."""

    def conjugated(self) -> typing_extensions.Self:
        """Return a new conjugated quaternion.

        :return: a new quaternion.
        :rtype: typing_extensions.Self
        """

    def copy(self) -> typing_extensions.Self:
        """Returns a copy of this quaternion.

        :return: A copy of the quaternion.
        :rtype: typing_extensions.Self
        """

    def cross(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """Return the cross product of this quaternion and another.

        :param other: The other quaternion to perform the cross product with.
        :type other: collections.abc.Sequence[float] | typing_extensions.Self
        :return: The cross product.
        :rtype: typing_extensions.Self
        """

    def dot(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> float:
        """Return the dot product of this quaternion and another.

        :param other: The other quaternion to perform the dot product with.
        :type other: collections.abc.Sequence[float] | typing_extensions.Self
        :return: The dot product.
        :rtype: float
        """

    def freeze(self) -> typing_extensions.Self:
        """Make this object immutable.After this the object can be hashed, used in dictionaries & sets.

        :return: An instance of this object.
        :rtype: typing_extensions.Self
        """

    def identity(self):
        """Set the quaternion to an identity quaternion."""

    def invert(self):
        """Set the quaternion to its inverse."""

    def inverted(self) -> typing_extensions.Self:
        """Return a new, inverted quaternion.

        :return: the inverted value.
        :rtype: typing_extensions.Self
        """

    def make_compatible(self, other):
        """Make this quaternion compatible with another,
        so interpolating between them works as intended.

                :param other:
        """

    def negate(self):
        """Set the quaternion to its negative."""

    def normalize(self):
        """Normalize the quaternion."""

    def normalized(self) -> typing_extensions.Self:
        """Return a new normalized quaternion.

        :return: a normalized copy.
        :rtype: typing_extensions.Self
        """

    def rotate(
        self,
        other: Euler
        | Matrix
        | collections.abc.Sequence[collections.abc.Sequence[float]]
        | collections.abc.Sequence[float]
        | typing_extensions.Self,
    ):
        """Rotates the quaternion by another mathutils value.

        :param other: rotation component of mathutils value
        :type other: Euler | Matrix | collections.abc.Sequence[collections.abc.Sequence[float]] | collections.abc.Sequence[float] | typing_extensions.Self
        """

    def rotation_difference(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """Returns a quaternion representing the rotational difference.

        :param other: second quaternion.
        :type other: collections.abc.Sequence[float] | typing_extensions.Self
        :return: the rotational difference between the two quat rotations.
        :rtype: typing_extensions.Self
        """

    def slerp(
        self,
        other: collections.abc.Sequence[float] | typing_extensions.Self,
        factor: float,
    ) -> typing_extensions.Self:
        """Returns the interpolation of two quaternions.

        :param other: value to interpolate with.
        :type other: collections.abc.Sequence[float] | typing_extensions.Self
        :param factor: The interpolation value in [0.0, 1.0].
        :type factor: float
        :return: The interpolated rotation.
        :rtype: typing_extensions.Self
        """

    def to_axis_angle(self) -> tuple[Vector, float]:
        """Return the axis, angle representation of the quaternion.

        :return: Axis, angle.
        :rtype: tuple[Vector, float]
        """

    def to_euler(
        self,
        order: str | None = "",
        euler_compat: Euler | collections.abc.Sequence[float] | None = [],
    ) -> Euler:
        """Return Euler representation of the quaternion.

                :param order: Optional rotation order argument in
        ['XYZ', 'XZY', 'YXZ', 'YZX', 'ZXY', 'ZYX'].
                :type order: str | None
                :param euler_compat: Optional euler argument the new euler will be made
        compatible with (no axis flipping between them).
        Useful for converting a series of matrices to animation curves.
                :type euler_compat: Euler | collections.abc.Sequence[float] | None
                :return: Euler representation of the quaternion.
                :rtype: Euler
        """

    def to_exponential_map(self):
        """Return the exponential map representation of the quaternion.This representation consist of the rotation axis multiplied by the rotation angle.
        Such a representation is useful for interpolation between multiple orientations.To convert back to a quaternion, pass it to the `Quaternion` constructor.

                :return: exponential map.
        """

    def to_matrix(self) -> Matrix:
        """Return a matrix representation of the quaternion.

        :return: A 3x3 rotation matrix representation of the quaternion.
        :rtype: Matrix
        """

    def to_swing_twist(self, axis: str) -> tuple[Quaternion, float]:
        """Split the rotation into a swing quaternion with the specified
        axis fixed at zero, and the remaining twist rotation angle.

                :param axis: Twist axis as a string in ['X', 'Y', 'Z'].
                :type axis: str
                :return: Swing, twist angle.
                :rtype: tuple[Quaternion, float]
        """

    def __init__(
        self,
        seq: Vector | collections.abc.Sequence[float] = (1.0, 0.0, 0.0, 0.0),
        angle: float = 0.0,
    ):
        """

        :param seq:
        :type seq: Vector | collections.abc.Sequence[float]
        :param angle:
        :type angle: float
        """

    def __get__(self, instance, owner) -> typing_extensions.Self:
        """

        :param instance:
        :param owner:
        :return:
        :rtype: typing_extensions.Self
        """

    def __set__(
        self, instance, value: collections.abc.Sequence[float] | typing_extensions.Self
    ):
        """

        :param instance:
        :param value:
        :type value: collections.abc.Sequence[float] | typing_extensions.Self
        """

    def __len__(self) -> int:
        """

        :return:
        :rtype: int
        """

    @typing.overload
    def __getitem__(self, key: int) -> float:
        """

        :param key:
        :type key: int
        :return:
        :rtype: float
        """

    @typing.overload
    def __getitem__(self, key: slice) -> tuple[float, ...]:
        """

        :param key:
        :type key: slice
        :return:
        :rtype: tuple[float, ...]
        """

    @typing.overload
    def __setitem__(self, key: int, value: float):
        """

        :param key:
        :type key: int
        :param value:
        :type value: float
        """

    @typing.overload
    def __setitem__(
        self, key: slice, value: collections.abc.Iterable[float] | Quaternion
    ):
        """

        :param key:
        :type key: slice
        :param value:
        :type value: collections.abc.Iterable[float] | Quaternion
        """

    def __add__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :type other: collections.abc.Sequence[float] | typing_extensions.Self
        :return:
        :rtype: typing_extensions.Self
        """

    def __sub__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :type other: collections.abc.Sequence[float] | typing_extensions.Self
        :return:
        :rtype: typing_extensions.Self
        """

    def __mul__(
        self, other: collections.abc.Sequence[float] | float | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :type other: collections.abc.Sequence[float] | float | typing_extensions.Self
        :return:
        :rtype: typing_extensions.Self
        """

    @typing.overload
    def __matmul__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :type other: collections.abc.Sequence[float] | typing_extensions.Self
        :return:
        :rtype: typing_extensions.Self
        """

    @typing.overload
    def __matmul__(self, other: Vector | collections.abc.Sequence[float]) -> Vector:
        """

        :param other:
        :type other: Vector | collections.abc.Sequence[float]
        :return:
        :rtype: Vector
        """

    def __radd__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :type other: collections.abc.Sequence[float] | typing_extensions.Self
        :return:
        :rtype: typing_extensions.Self
        """

    def __rsub__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :type other: collections.abc.Sequence[float] | typing_extensions.Self
        :return:
        :rtype: typing_extensions.Self
        """

    def __rmul__(
        self, other: collections.abc.Sequence[float] | float | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :type other: collections.abc.Sequence[float] | float | typing_extensions.Self
        :return:
        :rtype: typing_extensions.Self
        """

    def __imul__(
        self, other: collections.abc.Sequence[float] | float | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :type other: collections.abc.Sequence[float] | float | typing_extensions.Self
        :return:
        :rtype: typing_extensions.Self
        """

class Vector:
    """This object gives access to Vectors in Blender."""

    is_frozen: bool
    """ True when this object has been frozen (read-only).

    :type: bool
    """

    is_valid: bool
    """ True when the owner of this data is valid.

    :type: bool
    """

    is_wrapped: bool
    """ True when this object wraps external data (read-only).

    :type: bool
    """

    length: float
    """ Vector Length.

    :type: float
    """

    length_squared: float
    """ Vector length squared (v.dot(v)).

    :type: float
    """

    magnitude: float
    """ Vector Length.

    :type: float
    """

    owner: typing.Any
    """ The item this is wrapping or None  (read-only)."""

    w: float
    """ Vector W axis (4D Vectors only).

    :type: float
    """

    ww: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    www: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wwww: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wwwx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wwwy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wwwz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wwx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wwxw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wwxx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wwxy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wwxz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wwy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wwyw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wwyx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wwyy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wwyz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wwz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wwzw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wwzx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wwzy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wwzz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wxw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wxww: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wxwx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wxwy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wxwz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wxx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wxxw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wxxx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wxxy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wxxz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wxy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wxyw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wxyx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wxyy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wxyz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wxz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wxzw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wxzx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wxzy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wxzz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wyw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wyww: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wywx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wywy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wywz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wyx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wyxw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wyxx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wyxy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wyxz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wyy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wyyw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wyyx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wyyy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wyyz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wyz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wyzw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wyzx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wyzy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wyzz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wzw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wzww: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wzwx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wzwy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wzwz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wzx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wzxw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wzxx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wzxy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wzxz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wzy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wzyw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wzyx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wzyy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wzyz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wzz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wzzw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wzzx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wzzy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    wzzz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    x: float
    """ Vector X axis.

    :type: float
    """

    xw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xww: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xwww: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xwwx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xwwy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xwwz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xwx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xwxw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xwxx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xwxy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xwxz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xwy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xwyw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xwyx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xwyy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xwyz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xwz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xwzw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xwzx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xwzy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xwzz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xxw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xxww: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xxwx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xxwy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xxwz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xxx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xxxw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xxxx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xxxy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xxxz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xxy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xxyw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xxyx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xxyy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xxyz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xxz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xxzw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xxzx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xxzy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xxzz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xyw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xyww: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xywx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xywy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xywz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xyx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xyxw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xyxx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xyxy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xyxz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xyy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xyyw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xyyx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xyyy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xyyz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xyz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xyzw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xyzx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xyzy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xyzz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xzw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xzww: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xzwx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xzwy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xzwz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xzx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xzxw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xzxx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xzxy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xzxz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xzy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xzyw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xzyx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xzyy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xzyz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xzz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xzzw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xzzx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xzzy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    xzzz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    y: float
    """ Vector Y axis.

    :type: float
    """

    yw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yww: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    ywww: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    ywwx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    ywwy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    ywwz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    ywx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    ywxw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    ywxx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    ywxy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    ywxz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    ywy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    ywyw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    ywyx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    ywyy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    ywyz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    ywz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    ywzw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    ywzx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    ywzy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    ywzz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yxw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yxww: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yxwx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yxwy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yxwz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yxx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yxxw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yxxx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yxxy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yxxz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yxy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yxyw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yxyx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yxyy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yxyz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yxz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yxzw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yxzx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yxzy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yxzz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yyw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yyww: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yywx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yywy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yywz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yyx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yyxw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yyxx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yyxy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yyxz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yyy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yyyw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yyyx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yyyy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yyyz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yyz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yyzw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yyzx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yyzy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yyzz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yzw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yzww: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yzwx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yzwy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yzwz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yzx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yzxw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yzxx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yzxy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yzxz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yzy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yzyw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yzyx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yzyy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yzyz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yzz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yzzw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yzzx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yzzy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    yzzz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    z: float
    """ Vector Z axis (3D Vectors only).

    :type: float
    """

    zw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zww: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zwww: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zwwx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zwwy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zwwz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zwx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zwxw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zwxx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zwxy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zwxz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zwy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zwyw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zwyx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zwyy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zwyz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zwz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zwzw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zwzx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zwzy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zwzz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zxw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zxww: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zxwx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zxwy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zxwz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zxx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zxxw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zxxx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zxxy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zxxz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zxy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zxyw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zxyx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zxyy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zxyz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zxz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zxzw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zxzx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zxzy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zxzz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zyw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zyww: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zywx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zywy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zywz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zyx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zyxw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zyxx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zyxy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zyxz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zyy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zyyw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zyyx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zyyy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zyyz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zyz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zyzw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zyzx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zyzy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zyzz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zzw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zzww: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zzwx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zzwy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zzwz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zzx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zzxw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zzxx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zzxy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zzxz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zzy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zzyw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zzyx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zzyy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zzyz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zzz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zzzw: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zzzx: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zzzy: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    zzzz: typing_extensions.Self
    """ 

    :type: typing_extensions.Self
    """

    @classmethod
    def Fill(cls, size: int, fill: float = 0.0):
        """Create a vector of length size with all values set to fill.

        :param size: The length of the vector to be created.
        :type size: int
        :param fill: The value used to fill the vector.
        :type fill: float
        """

    @classmethod
    def Linspace(cls, start: int, stop: int, size: int):
        """Create a vector of the specified size which is filled with linearly spaced values between start and stop values.

        :param start: The start of the range used to fill the vector.
        :type start: int
        :param stop: The end of the range used to fill the vector.
        :type stop: int
        :param size: The size of the vector to be created.
        :type size: int
        """

    @classmethod
    def Range(cls, start: int, stop: int, step: int = 1):
        """Create a filled with a range of values.

        :param start: The start of the range used to fill the vector.
        :type start: int
        :param stop: The end of the range used to fill the vector.
        :type stop: int
        :param step: The step between successive values in the vector.
        :type step: int
        """

    @classmethod
    def Repeat(
        cls, vector: collections.abc.Sequence[float] | typing_extensions.Self, size: int
    ):
        """Create a vector by repeating the values in vector until the required size is reached.

        :param vector: The vector to draw values from.
        :type vector: collections.abc.Sequence[float] | typing_extensions.Self
        :param size: The size of the vector to be created.
        :type size: int
        """

    def angle(
        self,
        other: collections.abc.Sequence[float] | typing_extensions.Self,
        fallback: typing.Any | None = None,
    ) -> float | typing.Any:
        """Return the angle between two vectors.

                :param other: another vector to compare the angle with
                :type other: collections.abc.Sequence[float] | typing_extensions.Self
                :param fallback: return this when the angle can't be calculated (zero length vector),
        (instead of raising a `ValueError`).
                :type fallback: typing.Any | None
                :return: angle in radians or fallback when given
                :rtype: float | typing.Any
        """

    def angle_signed(
        self,
        other: collections.abc.Sequence[float] | typing_extensions.Self,
        fallback: typing.Any | None = None,
    ) -> float | typing.Any:
        """Return the signed angle between two 2D vectors (clockwise is positive).

                :param other: another vector to compare the angle with
                :type other: collections.abc.Sequence[float] | typing_extensions.Self
                :param fallback: return this when the angle can't be calculated (zero length vector),
        (instead of raising a `ValueError`).
                :type fallback: typing.Any | None
                :return: angle in radians or fallback when given
                :rtype: float | typing.Any
        """

    def copy(self) -> typing_extensions.Self:
        """Returns a copy of this vector.

        :return: A copy of the vector.
        :rtype: typing_extensions.Self
        """

    def cross(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> float | typing_extensions.Self:
        """Return the cross product of this vector and another.

        :param other: The other vector to perform the cross product with.
        :type other: collections.abc.Sequence[float] | typing_extensions.Self
        :return: The cross product as a vector or a float when 2D vectors are used.
        :rtype: float | typing_extensions.Self
        """

    def dot(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> float:
        """Return the dot product of this vector and another.

        :param other: The other vector to perform the dot product with.
        :type other: collections.abc.Sequence[float] | typing_extensions.Self
        :return: The dot product.
        :rtype: float
        """

    def freeze(self) -> typing_extensions.Self:
        """Make this object immutable.After this the object can be hashed, used in dictionaries & sets.

        :return: An instance of this object.
        :rtype: typing_extensions.Self
        """

    def lerp(
        self,
        other: collections.abc.Sequence[float] | typing_extensions.Self,
        factor: float,
    ) -> typing_extensions.Self:
        """Returns the interpolation of two vectors.

        :param other: value to interpolate with.
        :type other: collections.abc.Sequence[float] | typing_extensions.Self
        :param factor: The interpolation value in [0.0, 1.0].
        :type factor: float
        :return: The interpolated vector.
        :rtype: typing_extensions.Self
        """

    def negate(self):
        """Set all values to their negative."""

    def normalize(self):
        """Normalize the vector, making the length of the vector always 1.0."""

    def normalized(self) -> typing_extensions.Self:
        """Return a new, normalized vector.

        :return: a normalized copy of the vector
        :rtype: typing_extensions.Self
        """

    def orthogonal(self) -> typing_extensions.Self:
        """Return a perpendicular vector.

        :return: a new vector 90 degrees from this vector.
        :rtype: typing_extensions.Self
        """

    def project(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """Return the projection of this vector onto the other.

        :param other: second vector.
        :type other: collections.abc.Sequence[float] | typing_extensions.Self
        :return: the parallel projection vector
        :rtype: typing_extensions.Self
        """

    def reflect(
        self, mirror: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """Return the reflection vector from the mirror argument.

        :param mirror: This vector could be a normal from the reflecting surface.
        :type mirror: collections.abc.Sequence[float] | typing_extensions.Self
        :return: The reflected vector matching the size of this vector.
        :rtype: typing_extensions.Self
        """

    def resize(self, size=3):
        """Resize the vector to have size number of elements.

        :param size:
        """

    def resize_2d(self):
        """Resize the vector to 2D  (x, y)."""

    def resize_3d(self):
        """Resize the vector to 3D  (x, y, z)."""

    def resize_4d(self):
        """Resize the vector to 4D (x, y, z, w)."""

    def resized(self, size=3) -> typing_extensions.Self:
        """Return a resized copy of the vector with size number of elements.

        :param size:
        :return: a new vector
        :rtype: typing_extensions.Self
        """

    def rotate(
        self,
        other: Euler
        | Matrix
        | Quaternion
        | collections.abc.Sequence[collections.abc.Sequence[float]]
        | collections.abc.Sequence[float],
    ):
        """Rotate the vector by a rotation value.

        :param other: rotation component of mathutils value
        :type other: Euler | Matrix | Quaternion | collections.abc.Sequence[collections.abc.Sequence[float]] | collections.abc.Sequence[float]
        """

    def rotation_difference(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> Quaternion:
        """Returns a quaternion representing the rotational difference between this
        vector and another.

                :param other: second vector.
                :type other: collections.abc.Sequence[float] | typing_extensions.Self
                :return: the rotational difference between the two vectors.
                :rtype: Quaternion
        """

    def slerp(
        self,
        other: collections.abc.Sequence[float] | typing_extensions.Self,
        factor: float,
        fallback: typing.Any | None = None,
    ) -> typing_extensions.Self:
        """Returns the interpolation of two non-zero vectors (spherical coordinates).

                :param other: value to interpolate with.
                :type other: collections.abc.Sequence[float] | typing_extensions.Self
                :param factor: The interpolation value typically in [0.0, 1.0].
                :type factor: float
                :param fallback: return this when the vector can't be calculated (zero length vector or direct opposites),
        (instead of raising a `ValueError`).
                :type fallback: typing.Any | None
                :return: The interpolated vector.
                :rtype: typing_extensions.Self
        """

    def to_2d(self) -> typing_extensions.Self:
        """Return a 2d copy of the vector.

        :return: a new vector
        :rtype: typing_extensions.Self
        """

    def to_3d(self) -> typing_extensions.Self:
        """Return a 3d copy of the vector.

        :return: a new vector
        :rtype: typing_extensions.Self
        """

    def to_4d(self) -> typing_extensions.Self:
        """Return a 4d copy of the vector.

        :return: a new vector
        :rtype: typing_extensions.Self
        """

    def to_track_quat(self, track: str, up: str) -> Quaternion:
        """Return a quaternion rotation from the vector and the track and up axis.

        :param track: Track axis in ['X', 'Y', 'Z', '-X', '-Y', '-Z'].
        :type track: str
        :param up: Up axis in ['X', 'Y', 'Z'].
        :type up: str
        :return: rotation from the vector and the track and up axis.
        :rtype: Quaternion
        """

    def to_tuple(self, precision: int = -1) -> tuple[float, ...]:
        """Return this vector as a tuple with.

        :param precision: The number to round the value to in [-1, 21].
        :type precision: int
        :return: the values of the vector rounded by precision
        :rtype: tuple[float, ...]
        """

    def zero(self):
        """Set all values to zero."""

    def __init__(self, seq: collections.abc.Sequence[float] = (0.0, 0.0, 0.0)):
        """

        :param seq:
        :type seq: collections.abc.Sequence[float]
        """

    def __get__(self, instance, owner) -> typing_extensions.Self:
        """

        :param instance:
        :param owner:
        :return:
        :rtype: typing_extensions.Self
        """

    def __set__(
        self, instance, value: collections.abc.Sequence[float] | typing_extensions.Self
    ):
        """

        :param instance:
        :param value:
        :type value: collections.abc.Sequence[float] | typing_extensions.Self
        """

    def __len__(self) -> int:
        """

        :return:
        :rtype: int
        """

    @typing.overload
    def __getitem__(self, key: int) -> float:
        """

        :param key:
        :type key: int
        :return:
        :rtype: float
        """

    @typing.overload
    def __getitem__(self, key: slice) -> tuple[float, ...]:
        """

        :param key:
        :type key: slice
        :return:
        :rtype: tuple[float, ...]
        """

    @typing.overload
    def __setitem__(self, key: int, value: float):
        """

        :param key:
        :type key: int
        :param value:
        :type value: float
        """

    @typing.overload
    def __setitem__(self, key: slice, value: collections.abc.Iterable[float] | Vector):
        """

        :param key:
        :type key: slice
        :param value:
        :type value: collections.abc.Iterable[float] | Vector
        """

    def __neg__(self) -> typing_extensions.Self:
        """

        :return:
        :rtype: typing_extensions.Self
        """

    def __add__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :type other: collections.abc.Sequence[float] | typing_extensions.Self
        :return:
        :rtype: typing_extensions.Self
        """

    def __sub__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :type other: collections.abc.Sequence[float] | typing_extensions.Self
        :return:
        :rtype: typing_extensions.Self
        """

    @typing.overload
    def __mul__(self, other: float) -> typing_extensions.Self:
        """

        :param other:
        :type other: float
        :return:
        :rtype: typing_extensions.Self
        """

    @typing.overload
    def __mul__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :type other: collections.abc.Sequence[float] | typing_extensions.Self
        :return:
        :rtype: typing_extensions.Self
        """

    def __truediv__(self, other: float) -> typing_extensions.Self:
        """

        :param other:
        :type other: float
        :return:
        :rtype: typing_extensions.Self
        """

    @typing.overload
    def __matmul__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> float:
        """

        :param other:
        :type other: collections.abc.Sequence[float] | typing_extensions.Self
        :return:
        :rtype: float
        """

    @typing.overload
    def __matmul__(
        self, other: Matrix | collections.abc.Sequence[collections.abc.Sequence[float]]
    ) -> typing_extensions.Self:
        """

        :param other:
        :type other: Matrix | collections.abc.Sequence[collections.abc.Sequence[float]]
        :return:
        :rtype: typing_extensions.Self
        """

    def __radd__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :type other: collections.abc.Sequence[float] | typing_extensions.Self
        :return:
        :rtype: typing_extensions.Self
        """

    def __rsub__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :type other: collections.abc.Sequence[float] | typing_extensions.Self
        :return:
        :rtype: typing_extensions.Self
        """

    def __rmul__(self, other: float) -> typing_extensions.Self:
        """

        :param other:
        :type other: float
        :return:
        :rtype: typing_extensions.Self
        """

    def __rtruediv__(self, other: float) -> typing_extensions.Self:
        """

        :param other:
        :type other: float
        :return:
        :rtype: typing_extensions.Self
        """

    def __iadd__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :type other: collections.abc.Sequence[float] | typing_extensions.Self
        :return:
        :rtype: typing_extensions.Self
        """

    def __isub__(
        self, other: collections.abc.Sequence[float] | typing_extensions.Self
    ) -> typing_extensions.Self:
        """

        :param other:
        :type other: collections.abc.Sequence[float] | typing_extensions.Self
        :return:
        :rtype: typing_extensions.Self
        """

    def __imul__(self, other: float) -> typing_extensions.Self:
        """

        :param other:
        :type other: float
        :return:
        :rtype: typing_extensions.Self
        """

    def __itruediv__(self, other: float) -> typing_extensions.Self:
        """

        :param other:
        :type other: float
        :return:
        :rtype: typing_extensions.Self
        """



"""
The message bus system can be used to receive notifications when properties of
Blender datablocks are changed via the data API.


--------------------

The message bus system is triggered by updates via the RNA system. This means
that the following updates will result in a notification on the message bus:

* Changes via the Python API, for example some_object.location.x += 3

.
* Changes via the sliders, fields, and buttons in the user interface.

The following updates do not trigger message bus notifications:

* Moving objects in the 3D Viewport.
* Changes performed by the animation system.


--------------------

Below is an example of subscription to changes in the active object's location.

```../examples/bpy.msgbus.1.py```

Some properties are converted to Python objects when you retrieve them. This
needs to be avoided in order to create the subscription, by using
datablock.path_resolve("property_name", False)

:

```../examples/bpy.msgbus.2.py```

It is also possible to create subscriptions on a property of all instances of a
certain type:

```../examples/bpy.msgbus.3.py```

[NOTE]
All subscribers will be cleared on file-load. Subscribers can be re-registered on load,
see bpy.app.handlers.load_post.

"""

import typing
import collections.abc
import typing_extensions
import bpy.types

def clear_by_owner(owner):
    """Clear all subscribers using this owner."""

def publish_rna(
    key: bpy.types.Property | bpy.types.Struct | tuple[bpy.types.Struct, str] | None,
):
    """Notify subscribers of changes to this property
    (this typically doesn't need to be called explicitly since changes will automatically publish updates).
    In some cases it may be useful to publish changes explicitly using more general keys.

        :param key: Represents the type of data being subscribed to

    Arguments include
    - A property instance.
    - A struct type.
    - A tuple representing a (struct, property name) pair.
        :type key: bpy.types.Property | bpy.types.Struct | tuple[bpy.types.Struct, str] | None
    """

def subscribe_rna(
    key: bpy.types.Property | bpy.types.Struct | tuple[bpy.types.Struct, str] | None,
    owner: typing.Any | None,
    args,
    notify,
    options=set(),
):
    """Register a message bus subscription. It will be cleared when another blend file is
    loaded, or can be cleared explicitly via `bpy.msgbus.clear_by_owner`.

        :param key: Represents the type of data being subscribed to

    Arguments include
    - A property instance.
    - A struct type.
    - A tuple representing a (struct, property name) pair.
        :type key: bpy.types.Property | bpy.types.Struct | tuple[bpy.types.Struct, str] | None
        :param owner: Handle for this subscription (compared by identity).
        :type owner: typing.Any | None
        :param options: Change the behavior of the subscriber.

    PERSISTENT when set, the subscriber will be kept when remapping ID data.
    """



import typing
import collections.abc
import typing_extensions

def execute_preset(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    filepath: str = "",
    menu_idname: str = "",
):
    """Load a preset

    :type execution_context: int | str | None
    :type undo: bool | None
    :param filepath: filepath
    :type filepath: str
    :param menu_idname: Menu ID Name, ID name of the menu this was called from
    :type menu_idname: str
    """

def python_file_run(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    filepath: str = "",
):
    """Run Python file

    :type execution_context: int | str | None
    :type undo: bool | None
    :param filepath: Path
    :type filepath: str
    """

def reload(execution_context: int | str | None = None, undo: bool | None = None):
    """Reload scripts

    :type execution_context: int | str | None
    :type undo: bool | None
    """



import typing
import collections.abc
import typing_extensions
import bpy._typing.rna_enums
import bpy.types

def actuator_add(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    type: str | None = "",
    name: str = "",
    object: str = "",
):
    """Add an actuator to the active object

    :type execution_context: int | str | None
    :type undo: bool | None
    :param type: Type, Type of actuator to add
    :type type: str | None
    :param name: Name, Name of the Actuator to add
    :type name: str
    :param object: Object, Name of the Object to add the Actuator to
    :type object: str
    """

def actuator_move(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    actuator: str = "",
    object: str = "",
    direction: typing.Literal["UP", "DOWN"] | None = "UP",
):
    """Move Actuator

    :type execution_context: int | str | None
    :type undo: bool | None
    :param actuator: Actuator, Name of the actuator to edit
    :type actuator: str
    :param object: Object, Name of the object the actuator belongs to
    :type object: str
    :param direction: Direction, Move Up or Down
    :type direction: typing.Literal['UP','DOWN'] | None
    """

def actuator_remove(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    actuator: str = "",
    object: str = "",
):
    """Remove an actuator from the active object

    :type execution_context: int | str | None
    :type undo: bool | None
    :param actuator: Actuator, Name of the actuator to edit
    :type actuator: str
    :param object: Object, Name of the object the actuator belongs to
    :type object: str
    """

def controller_add(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    type: bpy._typing.rna_enums.ControllerTypeItems | None = "LOGIC_AND",
    name: str = "",
    object: str = "",
):
    """Add a controller to the active object

    :type execution_context: int | str | None
    :type undo: bool | None
    :param type: Type, Type of controller to add
    :type type: bpy._typing.rna_enums.ControllerTypeItems | None
    :param name: Name, Name of the Controller to add
    :type name: str
    :param object: Object, Name of the Object to add the Controller to
    :type object: str
    """

def controller_move(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    controller: str = "",
    object: str = "",
    direction: typing.Literal["UP", "DOWN"] | None = "UP",
):
    """Move Controller

    :type execution_context: int | str | None
    :type undo: bool | None
    :param controller: Controller, Name of the controller to edit
    :type controller: str
    :param object: Object, Name of the object the controller belongs to
    :type object: str
    :param direction: Direction, Move Up or Down
    :type direction: typing.Literal['UP','DOWN'] | None
    """

def controller_remove(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    controller: str = "",
    object: str = "",
):
    """Remove a controller from the active object

    :type execution_context: int | str | None
    :type undo: bool | None
    :param controller: Controller, Name of the controller to edit
    :type controller: str
    :param object: Object, Name of the object the controller belongs to
    :type object: str
    """

def custom_object_create(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    class_name: str = "module.MyObject",
):
    """Create a KX_GameObject subclass and attach it to the selected object

    :type execution_context: int | str | None
    :type undo: bool | None
    :param class_name: MyObject, The class name with module (module.ClassName)
    :type class_name: str
    """

def custom_object_register(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    class_name: str = "module.MyObject",
):
    """Use a custom KX_GameObject subclass for the selected object

    :type execution_context: int | str | None
    :type undo: bool | None
    :param class_name: MyObject, The class name with module (module.ClassName)
    :type class_name: str
    """

def custom_object_reload(
    execution_context: int | str | None = None, undo: bool | None = None
):
    """Reload custom object from the source script

    :type execution_context: int | str | None
    :type undo: bool | None
    """

def custom_object_remove(
    execution_context: int | str | None = None, undo: bool | None = None
):
    """Remove this custom class from the object

    :type execution_context: int | str | None
    :type undo: bool | None
    """

def links_cut(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    path: bpy.types.bpy_prop_collection[bpy.types.OperatorMousePath] | None = None,
    cursor: int | None = 15,
):
    """Remove logic brick connections

    :type execution_context: int | str | None
    :type undo: bool | None
    :param path: Path
    :type path: bpy.types.bpy_prop_collection[bpy.types.OperatorMousePath] | None
    :param cursor: Cursor
    :type cursor: int | None
    """

def properties(execution_context: int | str | None = None, undo: bool | None = None):
    """Toggle the properties region visibility

    :type execution_context: int | str | None
    :type undo: bool | None
    """

def python_component_create(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    component_name: str = "module.Component",
):
    """Create a Python component to the selected object

    :type execution_context: int | str | None
    :type undo: bool | None
    :param component_name: Component, The component class name with module (module.ComponentName)
    :type component_name: str
    """

def python_component_move_down(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    index: int | None = 0,
):
    """Move this component down in the list

    :type execution_context: int | str | None
    :type undo: bool | None
    :param index: Index, Component index to move
    :type index: int | None
    """

def python_component_move_up(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    index: int | None = 0,
):
    """Move this component up in the list

    :type execution_context: int | str | None
    :type undo: bool | None
    :param index: Index, Component index to move
    :type index: int | None
    """

def python_component_register(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    component_name: str = "module.Component",
):
    """Add a Python component to the selected object

    :type execution_context: int | str | None
    :type undo: bool | None
    :param component_name: Component, The component class name with module (module.ComponentName)
    :type component_name: str
    """

def python_component_reload(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    index: int | None = 0,
):
    """Reload component from the source script

    :type execution_context: int | str | None
    :type undo: bool | None
    :param index: Index, Component index to reload
    :type index: int | None
    """

def python_component_remove(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    index: int | None = 0,
):
    """Remove this component from the object

    :type execution_context: int | str | None
    :type undo: bool | None
    :param index: Index, Component index to remove
    :type index: int | None
    """

def region_flip(execution_context: int | str | None = None, undo: bool | None = None):
    """Toggle the properties region's alignment (left/right)

    :type execution_context: int | str | None
    :type undo: bool | None
    """

def sensor_add(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    type: str | None = "",
    name: str = "",
    object: str = "",
):
    """Add a sensor to the active object

    :type execution_context: int | str | None
    :type undo: bool | None
    :param type: Type, Type of sensor to add
    :type type: str | None
    :param name: Name, Name of the Sensor to add
    :type name: str
    :param object: Object, Name of the Object to add the Sensor to
    :type object: str
    """

def sensor_move(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    sensor: str = "",
    object: str = "",
    direction: typing.Literal["UP", "DOWN"] | None = "UP",
):
    """Move Sensor

    :type execution_context: int | str | None
    :type undo: bool | None
    :param sensor: Sensor, Name of the sensor to edit
    :type sensor: str
    :param object: Object, Name of the object the sensor belongs to
    :type object: str
    :param direction: Direction, Move Up or Down
    :type direction: typing.Literal['UP','DOWN'] | None
    """

def sensor_remove(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    sensor: str = "",
    object: str = "",
):
    """Remove a sensor from the active object

    :type execution_context: int | str | None
    :type undo: bool | None
    :param sensor: Sensor, Name of the sensor to edit
    :type sensor: str
    :param object: Object, Name of the object the sensor belongs to
    :type object: str
    """

def view_all(execution_context: int | str | None = None, undo: bool | None = None):
    """Resize view so you can see all logic bricks

    :type execution_context: int | str | None
    :type undo: bool | None
    """



import typing
import collections.abc
import typing_extensions
import bgui.widget

class TextInput(bgui.widget.Widget):
    """Widget for getting text input"""

    children: typing.Any
    on_active: typing.Any
    on_click: typing.Any
    on_enter_key: typing.Any
    on_hover: typing.Any
    on_mouse_enter: typing.Any
    on_mouse_exit: typing.Any
    on_release: typing.Any
    parent: typing.Any
    position: typing.Any
    prefix: typing.Any
    size: typing.Any
    system: typing.Any
    text: typing.Any
    theme_options: typing.Any
    theme_section: typing.Any

    def activate(self): ...
    def calc_mouse_cursor(self, pos):
        """

        :param pos:
        """

    def deactivate(self): ...
    def find_mouse_slice(self, pos):
        """

        :param pos:
        """

    def select_all(self):
        """Change the selection to include all of the text"""

    def select_none(self):
        """Change the selection to include none of the text"""

    def swapcolors(self, state=0):
        """

        :param state:
        """

    def update_selection(self): ...



import typing
import collections.abc
import typing_extensions
from . import add_mesh_torus as add_mesh_torus
from . import anim as anim
from . import assets as assets
from . import bmesh as bmesh
from . import bone_selection_sets as bone_selection_sets
from . import clip as clip
from . import connect_to_output as connect_to_output
from . import console as console
from . import constraint as constraint
from . import file as file
from . import freestyle as freestyle
from . import geometry_nodes as geometry_nodes
from . import grease_pencil as grease_pencil
from . import image as image
from . import image_as_planes as image_as_planes
from . import mesh as mesh
from . import node as node
from . import node_editor as node_editor
from . import object as object
from . import object_align as object_align
from . import object_quick_effects as object_quick_effects
from . import object_randomize_transform as object_randomize_transform
from . import presets as presets
from . import rigidbody as rigidbody
from . import screen_play_rendered_anim as screen_play_rendered_anim
from . import sequencer as sequencer
from . import spreadsheet as spreadsheet
from . import userpref as userpref
from . import uvcalc_follow_active as uvcalc_follow_active
from . import uvcalc_lightmap as uvcalc_lightmap
from . import uvcalc_transform as uvcalc_transform
from . import vertexpaint_dirt as vertexpaint_dirt
from . import view3d as view3d
from . import wm as wm
from . import world as world

def register(): ...
def unregister(): ...



"""
Miscellaneous math utilities module

"""

import typing
import collections.abc
import typing_extensions

def clamp(value: float, min: float = 0, max: float = 1) -> float:
    """Clamps the float value between minimum and maximum. To avoid
    confusion, any call must use either one or all three arguments.

        :param value: The value to clamp.
        :type value: float
        :param min: The minimum value, defaults to 0.
        :type min: float
        :param max: The maximum value, defaults to 1.
        :type max: float
        :return: The clamped value.
        :rtype: float
    """

def lerp(from_value: float, to_value: float, factor: float) -> float:
    """Linearly interpolate between two float values based on factor.

    :param from_value: The value to return when factor is 0.
    :type from_value: float
    :param to_value: The value to return when factor is 1.
    :type to_value: float
    :param factor: The interpolation value, normally in [0.0, 1.0].
    :type factor: float
    :return: The interpolated value.
    :rtype: float
    """

def smoothstep(from_value: float, to_value: float, value) -> float:
    """Performs smooth interpolation between 0 and 1 as value changes between from and to values.
    Outside the range the function returns the same value as the nearest edge.

        :param from_value: The edge value where the result is 0.
        :type from_value: float
        :param to_value: The edge value where the result is 1.
        :type to_value: float
        :return: The interpolated value in [0.0, 1.0].
        :rtype: float
    """



"""
This module provides access to the matrix stack.

"""

import typing
import collections.abc
import typing_extensions
import mathutils

def get_model_view_matrix() -> mathutils.Matrix:
    """Return a copy of the model-view matrix.

    :return: A 4x4 view matrix.
    :rtype: mathutils.Matrix
    """

def get_normal_matrix() -> mathutils.Matrix:
    """Return a copy of the normal matrix.

    :return: A 3x3 normal matrix.
    :rtype: mathutils.Matrix
    """

def get_projection_matrix() -> mathutils.Matrix:
    """Return a copy of the projection matrix.

    :return: A 4x4 projection matrix.
    :rtype: mathutils.Matrix
    """

def load_identity():
    """Load an identity matrix into the stack."""

def load_matrix(
    matrix: collections.abc.Sequence[collections.abc.Sequence[float]]
    | mathutils.Matrix,
):
    """Load a matrix into the stack.

    :param matrix: A 4x4 matrix.
    :type matrix: collections.abc.Sequence[collections.abc.Sequence[float]] | mathutils.Matrix
    """

def load_projection_matrix(
    matrix: collections.abc.Sequence[collections.abc.Sequence[float]]
    | mathutils.Matrix,
):
    """Load a projection matrix into the stack.

    :param matrix: A 4x4 matrix.
    :type matrix: collections.abc.Sequence[collections.abc.Sequence[float]] | mathutils.Matrix
    """

def multiply_matrix(
    matrix: collections.abc.Sequence[collections.abc.Sequence[float]]
    | mathutils.Matrix,
):
    """Multiply the current stack matrix.

    :param matrix: A 4x4 matrix.
    :type matrix: collections.abc.Sequence[collections.abc.Sequence[float]] | mathutils.Matrix
    """

def pop():
    """Remove the last model-view matrix from the stack."""

def pop_projection():
    """Remove the last projection matrix from the stack."""

def push():
    """Add to the model-view matrix stack."""

def push_pop():
    """Context manager to ensure balanced push/pop calls, even in the case of an error."""

def push_pop_projection():
    """Context manager to ensure balanced push/pop calls, even in the case of an error."""

def push_projection():
    """Add to the projection matrix stack."""

def reset():
    """Empty stack and set to identity."""

def scale(scale: collections.abc.Sequence[float]):
    """Scale the current stack matrix.

    :param scale: Scale the current stack matrix with 2 or 3 floats.
    :type scale: collections.abc.Sequence[float]
    """

def scale_uniform(scale: float):
    """

    :param scale: Scale the current stack matrix.
    :type scale: float
    """

def translate(offset: collections.abc.Sequence[float]):
    """Scale the current stack matrix.

    :param offset: Translate the current stack matrix with 2 or 3 floats.
    :type offset: collections.abc.Sequence[float]
    """



import typing
import collections.abc
import typing_extensions
import bl_ui.properties_grease_pencil_common
import bl_ui.space_toolsystem_common
import bl_ui.utils
import bpy.types

class NODE_HT_header(bpy.types.Header):
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def draw(self, context):
        """

        :param context:
        """

class NODE_MT_add(bpy.types.Menu):
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    bl_translation_context: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def draw(self, context):
        """

        :param context:
        """

class NODE_MT_context_menu(bpy.types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def draw(self, context):
        """

        :param context:
        """

class NODE_MT_context_menu_select_menu(bpy.types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def draw(self, context):
        """

        :param context:
        """

class NODE_MT_context_menu_show_hide_menu(bpy.types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def draw(self, context):
        """

        :param context:
        """

class NODE_MT_editor_menus(bpy.types.Menu):
    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def draw(self, _context):
        """

        :param _context:
        """

class NODE_MT_node(bpy.types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def draw(self, context):
        """

        :param context:
        """

class NODE_MT_node_color_context_menu(bpy.types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def draw(self, _context):
        """

        :param _context:
        """

class NODE_MT_node_tree_interface_context_menu(bpy.types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def draw(self, context):
        """

        :param context:
        """

class NODE_MT_select(bpy.types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def draw(self, _context):
        """

        :param _context:
        """

class NODE_MT_view(bpy.types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def draw(self, context):
        """

        :param context:
        """

class NODE_MT_view_pie(bpy.types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def draw(self, _context):
        """

        :param _context:
        """

class NODE_PT_active_node_color(bpy.types.Panel):
    bl_category: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_parent_id: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def draw(self, context):
        """

        :param context:
        """

    def draw_header(self, context):
        """

        :param context:
        """

    def draw_header_preset(self, _context):
        """

        :param _context:
        """

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """

class NODE_PT_active_node_generic(bpy.types.Panel):
    bl_category: typing.Any
    bl_label: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def draw(self, context):
        """

        :param context:
        """

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """

class NODE_PT_active_node_properties(bpy.types.Panel):
    bl_category: typing.Any
    bl_label: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def draw(self, context):
        """

        :param context:
        """

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """

class NODE_PT_active_tool(
    bl_ui.space_toolsystem_common.ToolActivePanelHelper, bpy.types.Panel
):
    bl_category: typing.Any
    bl_label: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

class NODE_PT_annotation(
    bl_ui.properties_grease_pencil_common.AnnotationDataPanel, bpy.types.Panel
):
    bl_category: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """

class NODE_PT_backdrop(bpy.types.Panel):
    bl_category: typing.Any
    bl_label: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def draw(self, context):
        """

        :param context:
        """

    def draw_header(self, context):
        """

        :param context:
        """

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """

class NODE_PT_geometry_node_tool_mode(bpy.types.Panel):
    bl_label: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    bl_ui_units_x: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def draw(self, context):
        """

        :param context:
        """

class NODE_PT_geometry_node_tool_object_types(bpy.types.Panel):
    bl_label: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    bl_ui_units_x: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def draw(self, context):
        """

        :param context:
        """

class NODE_PT_geometry_node_tool_options(bpy.types.Panel):
    bl_label: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    bl_ui_units_x: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def draw(self, context):
        """

        :param context:
        """

class NODE_PT_material_slots(bpy.types.Panel):
    bl_label: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    bl_ui_units_x: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def draw(self, context):
        """

        :param context:
        """

    def draw_header(self, context):
        """

        :param context:
        """

class NODE_PT_node_color_presets(bl_ui.utils.PresetPanel, bpy.types.Panel):
    """Predefined node color"""

    bl_label: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any
    preset_add_operator: typing.Any
    preset_operator: typing.Any
    preset_subdir: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

class NODE_PT_node_tree_interface(bpy.types.Panel):
    bl_category: typing.Any
    bl_label: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def draw(self, context):
        """

        :param context:
        """

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """

class NODE_PT_node_tree_interface_panel_toggle(bpy.types.Panel):
    bl_category: typing.Any
    bl_label: typing.Any
    bl_parent_id: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def draw(self, context):
        """

        :param context:
        """

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """

class NODE_PT_node_tree_properties(bpy.types.Panel):
    bl_category: typing.Any
    bl_label: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def draw(self, context):
        """

        :param context:
        """

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """

class NODE_PT_overlay(bpy.types.Panel):
    bl_label: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    bl_ui_units_x: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def draw(self, context):
        """

        :param context:
        """

class NODE_PT_quality(bpy.types.Panel):
    bl_category: typing.Any
    bl_label: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def draw(self, context):
        """

        :param context:
        """

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """

class NODE_PT_texture_mapping(bpy.types.Panel):
    COMPAT_ENGINES: typing.Any
    bl_category: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_region_type: typing.Any
    bl_rna: typing.Any
    bl_space_type: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def draw(self, context):
        """

        :param context:
        """

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """

def node_panel(cls): ...



import typing
import collections.abc
import typing_extensions
import bpy.types

class Fade:
    animated_property: typing.Any
    duration: typing.Any
    end: typing.Any
    max_value: typing.Any
    start: typing.Any
    type: typing.Any

    def calculate_max_value(self, sequence, fade_fcurve):
        """Returns the maximum Y coordinate the fade animation should use for a given sequence
        Uses either the sequence's value for the animated property, or the next keyframe after the fade

                :param sequence:
                :param fade_fcurve:
        """

class SequencerFileHandlerBase:
    @classmethod
    def poll_drop(cls, context):
        """

        :param context:
        """

class SequencerCrossfadeSounds(bpy.types.Operator):
    """Do cross-fading volume animation of two selected sound strips"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def execute(self, context):
        """

        :param context:
        """

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """

class SequencerDeinterlaceSelectedMovies(bpy.types.Operator):
    """Deinterlace all selected movie sources"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def execute(self, context):
        """

        :param context:
        """

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """

class SequencerFadesAdd(bpy.types.Operator):
    """Adds or updates a fade animation for either visual or audio strips"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def calculate_fade_duration(self, context, sequence):
        """

        :param context:
        :param sequence:
        """

    def calculate_fades(self, sequence, fade_fcurve, animated_property, duration):
        """Returns a list of Fade objects

        :param sequence:
        :param fade_fcurve:
        :param animated_property:
        :param duration:
        """

    def execute(self, context):
        """

        :param context:
        """

    def fade_animation_clear(self, fade_fcurve, fades):
        """Removes existing keyframes in the fades' time range, in fast mode, without
        updating the fcurve

                :param fade_fcurve:
                :param fades:
        """

    def fade_animation_create(self, fade_fcurve, fades):
        """Inserts keyframes in the fade_fcurve in fast mode using the Fade objects.
        Updates the fcurve after having inserted all keyframes to finish the animation.

                :param fade_fcurve:
                :param fades:
        """

    def fade_find_or_create_fcurve(self, context, sequence, animated_property):
        """Iterates over all the fcurves until it finds an fcurve with a data path
        that corresponds to the sequence.
        Returns the matching FCurve or creates a new one if the function can't find a match.

                :param context:
                :param sequence:
                :param animated_property:
        """

    def is_long_enough(self, sequence, duration=0.0):
        """

        :param sequence:
        :param duration:
        """

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """

class SequencerFadesClear(bpy.types.Operator):
    """Removes fade animation from selected sequences"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def execute(self, context):
        """

        :param context:
        """

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """

class SequencerSplitMulticam(bpy.types.Operator):
    """Split multicam strip and select camera"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def execute(self, context):
        """

        :param context:
        """

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """

class SEQUENCER_FH_image_strip(bpy.types.FileHandler, SequencerFileHandlerBase):
    bl_file_extensions: typing.Any
    bl_idname: typing.Any
    bl_import_operator: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

class SEQUENCER_FH_movie_strip(bpy.types.FileHandler, SequencerFileHandlerBase):
    bl_file_extensions: typing.Any
    bl_idname: typing.Any
    bl_import_operator: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

class SEQUENCER_FH_sound_strip(bpy.types.FileHandler, SequencerFileHandlerBase):
    bl_file_extensions: typing.Any
    bl_idname: typing.Any
    bl_import_operator: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

def calculate_duration_frames(scene, duration_seconds): ...



import typing
import collections.abc
import typing_extensions
import bpy.types

class NODE_FH_image_node(bpy.types.FileHandler):
    bl_file_extensions: typing.Any
    bl_idname: typing.Any
    bl_import_operator: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    @classmethod
    def poll_drop(cls, context):
        """

        :param context:
        """

class NODE_OT_add_foreach_geometry_element_zone(
    NodeAddZoneOperator, bpy.types.Operator
):
    """Add a For Each Geometry Element zone that allows executing nodes e.g. for each vertex separately"""

    add_default_geometry_link: typing.Any
    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any
    input_node_type: typing.Any
    output_node_type: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

class NODE_OT_add_node(NodeAddOperator, bpy.types.Operator):
    """Add a node to the active tree"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    @classmethod
    def description(cls, _context, properties):
        """

        :param _context:
        :param properties:
        """

    def execute(self, context):
        """

        :param context:
        """

class NODE_OT_add_repeat_zone(NodeAddZoneOperator, bpy.types.Operator):
    """Add a repeat zone that allows executing nodes a dynamic number of times"""

    add_default_geometry_link: typing.Any
    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any
    input_node_type: typing.Any
    output_node_type: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

class NODE_OT_add_simulation_zone(NodeAddZoneOperator, bpy.types.Operator):
    """Add simulation zone input and output nodes to the active tree"""

    add_default_geometry_link: typing.Any
    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any
    input_node_type: typing.Any
    output_node_type: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

class NODE_OT_collapse_hide_unused_toggle(bpy.types.Operator):
    """Toggle collapsed nodes and hide unused sockets"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def execute(self, context):
        """

        :param context:
        """

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """

class NODE_OT_interface_item_duplicate(NodeInterfaceOperator, bpy.types.Operator):
    """Add a copy of the active item to the interface"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def execute(self, context):
        """

        :param context:
        """

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """

class NODE_OT_interface_item_make_panel_toggle(
    NodeInterfaceOperator, bpy.types.Operator
):
    """Make the active boolean socket a toggle for its parent panel"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def execute(self, context):
        """

        :param context:
        """

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """

class NODE_OT_interface_item_new(NodeInterfaceOperator, bpy.types.Operator):
    """Add a new item to the interface"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def execute(self, context):
        """

        :param context:
        """

    @staticmethod
    def find_valid_socket_type(tree):
        """

        :param tree:
        """

    def get_items(self, _self, context):
        """

        :param _self:
        :param context:
        """

class NODE_OT_interface_item_remove(NodeInterfaceOperator, bpy.types.Operator):
    """Remove active item from the interface"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def execute(self, context):
        """

        :param context:
        """

class NODE_OT_interface_item_unlink_panel_toggle(
    NodeInterfaceOperator, bpy.types.Operator
):
    """Make the panel toggle a stand-alone socket"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def execute(self, context):
        """

        :param context:
        """

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """

class NODE_OT_tree_path_parent(bpy.types.Operator):
    """Go to parent node tree"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def execute(self, context):
        """

        :param context:
        """

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """

class NODE_OT_viewer_shortcut_get(bpy.types.Operator):
    """Activate a specific compositor viewer node using 1,2,..,9 keys"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def execute(self, context):
        """

        :param context:
        """

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """

class NODE_OT_viewer_shortcut_set(bpy.types.Operator):
    """Create a compositor viewer shortcut for the selected node by pressing ctrl+1,2,..9"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

    def execute(self, context):
        """

        :param context:
        """

    def get_connected_viewer(self, node):
        """

        :param node:
        """

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """

class NodeAddOperator:
    def create_node(self, context, node_type):
        """

        :param context:
        :param node_type:
        """

    @staticmethod
    def deselect_nodes(context):
        """

        :param context:
        """

    def invoke(self, context, event):
        """

        :param context:
        :param event:
        """

    @classmethod
    def poll(cls, context):
        """

        :param context:
        """

    @staticmethod
    def store_mouse_cursor(context, event):
        """

        :param context:
        :param event:
        """

class NodeInterfaceOperator:
    @classmethod
    def poll(cls, context):
        """

        :param context:
        """

class NodeSetting(bpy.types.PropertyGroup):
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        :rtype: bpy.types.Struct
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        :rtype: typing.Any
        """

class NodeAddZoneOperator(NodeAddOperator):
    add_default_geometry_link: typing.Any

    def execute(self, context):
        """

        :param context:
        """
